import requests
from requests import Response

from .instance import Instance


class HttpRequests:

    def __init__(self):
        self._instance = Instance()
        self._token = None

    @property
    def token(self):
        if not self._token:
            self._token = self._get_token()
            self._instance.headers["Authorization"] = "Bearer {}".format(self.token)
        return self._token

    def push_dataset(self, dataset) -> Response:
        return requests.post(self._dataset_url(),
                             headers=self._instance.headers,
                             json=dataset.metadata)

    def update_dataset(self, dataset) -> Response:
        return requests.patch(self._dataset_url_by_pid(dataset.pid),
                              headers=(self._instance.headers),
                              json=dataset.metadata)

    def get_dataset(self, pid: str) -> dict:

        # Why: old implementation added extra / to pid-url
        params = {
            'filter': '{"where": {"$or": [{"pid": "' + pid + '"}, {"pid": "/' + pid + '"}]}}'
        }

        response = requests.get(self._dataset_url(),
                                headers=self._instance.headers,
                                params=params)

        if response.ok:
            if not response.json():
                raise Exception(f"No dataset found for pid: {pid}")
            return response.json()
        else:
            raise Exception(f"{response.text}")

    def get_all_datasets(self, proposal_id: int) -> list:

        params = {
            'filter': '{"where": {"proposalId": "' + str(proposal_id) + '"}, "limits": {"limit": 5000, "order": "asc"}}'
        }

        response = requests.get(self._dataset_url(),
                                headers=self._instance.headers,
                                params=params)

        if response.ok:
            return [each for each in response.json()]
        else:
            raise Exception(f"{response.text}")

    #TODO: add image does not work probably URL with new backend
    def add_image_to_dataset(self, pid: str, image_request: str) -> Response:
        return requests.post(self._attachments_url_by_pid(pid),
                             headers=self._instance.headers,
                             data=image_request)

    def _get_token(self) -> str:
        url = f"{self._instance.scicat_url}/users/login"
        response = requests.post(url, headers=self._instance.headers,
                                 json={"username": self._instance.username,
                                       "password": self._instance.pw})
        if response.ok:
            return response.json()["id"]
        else:
            raise Exception(f"{response.text}")

    def _dataset_url(self) -> str:
        return f"{self._instance.scicat_url}/Datasets?access_token=Bearer%20{self.token}"

    def _dataset_url_by_pid(self, pid: str) -> str:
        npid = self._reformat_pid(pid)
        return f"{self._instance.scicat_url}/Datasets/{npid}?access_token=Bearer%20{self.token}"

    def _attachments_url_by_pid(self, pid: str) -> str:
        npid = self._reformat_pid(pid)
        return f"{self._instance.scicat_url}/Datasets/{npid}/Attachments?access_token=Bearer%20{self.token}"

    @staticmethod
    def _reformat_pid(pid: str) -> str:
        """ SciCat adds a pid-prefix + "/"  but at DESY prefix = "" """
        # return ("/" + pid).replace("/", "%2F")
        return (pid).replace("/", "%2F")
